MOV R0, #1
MOV R1, #2
MOV R2, #3
MOV R3, #4
MOV R4, #5
MOV R5, #6
MOV R6, #7
MOV R7, #8
MUL R8, R0, R4
MUL R9, R1, R6
ADD R4, R8, R9
MUL R8, R0, R5
MUL R9, R1, R7
ADD R5, R8, R9
MUL R8, R2, R4
MUL R9, R3, R6
ADD R6, R8, R9
MUL R8, R2, R5
MUL R9, R3, R7
ADD R7, R8, R9
ADD R8, R4, R5
ADD R9, R6, R7
ADD R0, R8, R9
MOV R10, #64
STR R0, [R10, #0]

